#menu-toggle {
	--border-color-default: transparent;
	margin-inline-end: -0.25rem;
	transition-property: border-color, background-color, color;
}

.mobile-sidebar-toggle #menu-toggle {
	--background-color-default: var(--theme-dim-lighter);
	--text-color-default: var(--text-color-focus);
	--border-color-default: var(--border-color-focus);
}

@media (min-width: 50em) {
	#menu-toggle {
		display: none;
	}
}
