.theme-toggle {
	display: inline-flex;
	align-items: center;
	gap: 0.25em;
	padding: 0.33em 0.67em;
	border-radius: 99em;
	background-color: var(--theme-code-inline-bg);
	/* Indicates the ThemeToggle boundaries for forced colors users, transparent is changed to a solid color */
	outline: 1px solid transparent;
}

.theme-toggle > label:focus-within {
	outline: 2px solid transparent;
	box-shadow:
		0 0 0 0.08em var(--theme-accent),
		0 0 0 0.12em white;
}

.theme-toggle > label {
	color: var(--theme-code-inline-text);
	position: relative;
	display: flex;
	align-items: center;
	justify-content: center;
	opacity: 0.5;
}

.theme-toggle .checked {
	color: var(--theme-accent);
	opacity: 1;
}

input[name="theme-toggle"] {
	position: absolute;
	opacity: 0;
	top: 0;
	right: 0;
	bottom: 0;
	left: 0;
	z-index: -1;
}
