.header-button {
	--border-color-default: var(--theme-shade-subtle);
	--border-color-hocus: var(--theme-text-light);
	--text-color-default: var(--theme-text-light);
	--text-color-hocus: var(--theme-text);
	box-sizing: border-box;
	margin: 0;
	padding: 0.8rem;
	overflow: visible;
	background-color: var(--background-color-default, transparent);
	border-style: solid;
	border-width: 1px;
	border-color: var(--border-color-default);
	border-radius: 0.25rem;
	color: var(--text-color-default);
	font-size: 1rem;
	font-family: inherit;
	font-weight: 500;
	line-height: 1;
	cursor: pointer;
	transition-timing-function: ease-out;
	transition-duration: 0.2s;
	transition-property: border-color;
	-webkit-font-smoothing: antialiased;
}

@media (hover: hover) {
	.header-button {
		padding: 0.58rem;
	}
	.header-button:hover {
		color: var(--text-color-hocus);
		border-color: var(--border-color-hocus);
	}
}
